#!/usr/bin/X11/tixwish
#  File: sync.tcl
# 
#      This file is part of minkowsky
# 
#      Copyright (C) 2001-2002 by Rdiger Goetz
#      Author: Rdiger Goetz <minkowsky@r-goetz.de>
# 
#      Time-stamp: <27-Oct-2002 20:28:24 goetz>
# 
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
# 
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
# 
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#  

proc initSyncPanel { } {
    global syncAddrCustomMax
    global syncDeviceParameter syncDeviceList syncDirection syncAdrFields
    global syncAddrBlockedFolder syncAddrBlockedCategory
    global syncAddrDefFolder syncAddrDefCategory
    global syncDevList
    global syncAddrTagsList syncDev

    set syncDevList "/dev/ttyS0 /dev/ttyS1 /dev/ttyS2 /dev/ttyS3 /dev/modem /dev/cua0 /dev/cua1"

    set syncAddrCustomMax 8

#     puts "getSyncParameters !!!!!!!!!!!!!!!"
    set ret [ request getSyncParameters ]
    if { $ret == "" || $ret == "Error" } { return }
    # ret = {list of devices} { {list of dev1 parameters} {list of dev2 parameters}  ... } {syncAdrFieldList }

    set syncDeviceList [lindex $ret 0]

    set list  [ lindex $ret 1 ]
    set i 0
    foreach dev $syncDeviceList {
	set syncDeviceParameter($dev) [lindex $list $i ]
	incr i
    }
    set list [lindex $ret 2]
    foreach ent $list {
	set id [lindex $ent 0]
	set syncAdrFields($id) [lindex $ent 1]
    }

    set syncDirection(adr) "inOut"
    set syncDirection(prj) "off"
    set syncDirection(cal) "off"

    # must be read from user prefs
    set syncAddrDefFolder "-"
    set syncAddrDefCategory "-"
    set syncAddrBlockedFolder "-"
    set syncAddrBlockedCategory "-"
    set syncAddrTagsList ""
    set syncDev "/dev/ttyS0"
}

proc drawSyncPanel { } {
    global syncDeviceList syncDevice
    global syncDevList syncDev
    global bfnt fnt bfnt12
    global syncDisConnCmdStr
    
    initSyncPanel
    set syncDisConnCmdStr ""
    
    
    toplevel .sync 
    
    wm title .sync "Syncronisation"
    
    frame .sync.t -relief raised -bd 1
    frame .sync.n -relief raised -bd 1
    frame .sync.b
    pack .sync.t .sync.n .sync.b -side top -fill x -ipadx 2 -ipady 2
    
    centerWin .sync 508 676
    
    frame .sync.t.1
    pack .sync.t.1 -side top -fill x 
    tixOptionMenu .sync.t.1.d  -options {menubutton.width 25 label.width 32} -label "Syncing PDA" 
    foreach  dev $syncDeviceList {
	.sync.t.1.d add command $dev -label $dev
    }
    .sync.t.1.d configure -variable syncDevice
    .sync.t.1.d subwidget menu configure -font $bfnt12
    .sync.t.1.d subwidget menubutton configure -font $fnt
    .sync.t.1.d subwidget label configure -font $bfnt
    pack .sync.t.1.d -side left -fill x -padx 3
    
    frame .sync.t.2
    pack .sync.t.2 -side top -fill x 
    tixOptionMenu .sync.t.2.d  -options {menubutton.width 25 label.width 32} -label "Syncing device" 
    foreach  dev $syncDevList {
	.sync.t.2.d add command $dev -label $dev
    }
    .sync.t.2.d configure -variable syncDevi
    .sync.t.2.d subwidget menu configure -font $bfnt12
    .sync.t.2.d subwidget menubutton configure -font $fnt
    .sync.t.2.d subwidget label configure -font $bfnt
    pack .sync.t.2.d -side left -fill x -padx 3
    
    tixNoteBook .sync.n.n -ipadx 3 -ipady 2
    pack .sync.n.n -side top -fill x
    
    .sync.n.n add cal  -label "Calendar" 
    .sync.n.n add proj -label "Task & Projects"  
    .sync.n.n add addr -label "Addresses"
    
    drawSyncCalPanel  [.sync.n.n subwidget cal]
    drawSyncProjPanel [.sync.n.n subwidget proj]
    drawSyncAddrPanel [.sync.n.n subwidget addr]
    
    button .sync.b.abb  -text "Cancel" -command {
	destroy .sync
    }
    button .sync.b.sync -text " Sync " -command {
	doSync
	destroy .sync
    }
    button .sync.b.pref -text " Save As Default " -command {saveSyncPrefs }

   
    pack .sync.b.abb -side left
    pack .sync.b.sync -side right
    pack .sync.b.pref -side right -padx 16
    .sync.n.n raise addr
    update

#    selectSyncPDAdevice [.sync.n.n subwidget cal] [.sync.n.n subwidget proj] [.sync.n.n subwidget addr] $syncDevice 
    readSyncPrefs
    update
    if { [winfo exists .sync ] } {
	.sync.t.1.d  configure -command {selectSyncPDAdevice [.sync.n.n subwidget cal] [.sync.n.n subwidget proj] [.sync.n.n subwidget addr]}
    }

    if { [winfo exists .sync ] } {
# 	puts "waiting for .sync to be removed"
	tkwait window  .sync
    }
    if {$syncDisConnCmdStr != "" } {
	global syncDisConnCmdStr
# 	puts ">> disconn-Cmd='$syncDisConnCmdStr'"
	set cmd [syncGetCommandStr $syncDisConnCmdStr ]
# 	puts "-> '$cmd'"
	eval $cmd 
    } 
    
}

proc drawSyncCalPanel { win } {
    global syncDirection
    global fnt bfnt bfnt12

    for { set i 1 } {$i < 2 } { incr i } {
	frame $win.$i
	pack $win.$i -side top -fill x
    }
    
    tixOptionMenu $win.1.d  -options {menubutton.width 25 label.width 25} -label "Syncing direction"
    $win.1.d add command into    -label "Sync into Minkowsky" -state disabled
    $win.1.d add command out   -label "Sync into PDA" -state disabled
    $win.1.d add command inout -label "Sync in and out" -state disabled
    $win.1.d add command off   -label "don't Sync"
    $win.1.d subwidget menu configure -font $bfnt12
    $win.1.d subwidget menubutton configure -font $fnt
    $win.1.d subwidget label configure -font $bfnt
    pack $win.1.d  -side top -fill x
    $win.1.d configure -variable syncDirection(cal)
}

proc drawSyncProjPanel { win } {
    global syncDirection
    global fnt bfnt bfnt12

    for { set i 1 } {$i < 2 } { incr i } {
	frame $win.$i
	pack $win.$i -side top -fill x
    }
    
    tixOptionMenu $win.1.d  -options {menubutton.width 25 label.width 25} -label "Syncing direction"
    $win.1.d add command into  -label "Sync into Minkowsky" -state disabled
    $win.1.d add command out   -label "Sync into PDA" -state disabled
    $win.1.d add command inOut -label "Sync in and out" -state disabled
    $win.1.d add command off   -label "don't Sync"
    $win.1.d subwidget menu configure -font $bfnt12
    $win.1.d subwidget menubutton configure -font $fnt
    $win.1.d subwidget label configure -font $bfnt
    pack $win.1.d  -side top -fill x
    $win.1.d configure -variable syncDirection(prj)
}


proc drawSyncAddrPanel { win } {
    global syncDirection
    global syncAddrDefFolder  syncAddrDefCategory syncAdrFields syncAddrBlockedFolder syncAddrBlockedCategory
    global syncAddrCustomMax
    global fnt bfnt bfnt12 fnt12 fnt10 bfnt10 fnt8
    global syncAdrBaseWin syncAdrSelectAdrFieldBaseWin 

    for { set i 1 } {$i < 12 } { incr i } {
	frame $win.$i -bd 2
	pack $win.$i -side top -fill x
    }
    set syncAdrBaseWin $win
    
    tixOptionMenu $win.1.d   -label "Syncing direction"
    $win.1.d add command into  -label "Sync into Minkowsky"
    $win.1.d add command out   -label "Sync into PDA"
    $win.1.d add command inOut -label "Sync in and out"
    $win.1.d add command off   -label "don't Sync"
    $win.1.d subwidget menu configure -font $bfnt12
    $win.1.d subwidget menubutton configure -font $fnt -width 16
    $win.1.d subwidget label configure -font $bfnt -width 25
    pack $win.1.d  -side top -fill x
    $win.1.d configure -variable syncDirection(adr)

    label $win.2.l -text "Folder (Minkowsky) settings" -font $bfnt12
    pack $win.2.l -side left -fill x
    $win.3 configure -relief ridge 
    frame $win.3.1
    frame $win.3.2
    pack $win.3.1 $win.3.2 -side top -fill x 

    label  $win.3.1.f -text "Default: "  -font $bfnt -width 10
    entry  $win.3.1.e -textvariable syncAddrDefFolder  -font $fnt -width 40 
    button $win.3.1.s -text "Select" -command {adrDirListWin sync-def} -font $bfnt
    pack $win.3.1.f $win.3.1.e $win.3.1.s -side left -fill x

    label  $win.3.2.f -text "Blocked: "  -font $bfnt -width 10
    label  $win.3.2.e -text $syncAddrBlockedFolder  -font $fnt -width 40
    button $win.3.2.s -text "Select" -command {adrDirListWin sync-block} -font $bfnt
    pack $win.3.2.f -side left -fill x -padx 1
    pack $win.3.2.e -side left -fill x -padx 3
    pack $win.3.2.s -side left -fill x -padx 0
    fillSyncAdrSetBlockedFolders

    label $win.4.l -text " Category (PDA) settings" -font $bfnt12
    pack $win.4.l -side left -fill x
    $win.5 configure -relief ridge 
    frame $win.5.1
    frame $win.5.2
    pack $win.5.1 $win.5.2 -side top -fill x 

    label  $win.5.1.f -text "Default: "  -font $bfnt -width 10
    label  $win.5.1.e -text $syncAddrDefCategory -font $fnt -width 40
    button $win.5.1.s -text "Select" -command {selectPDAcategories def} -font $bfnt
    pack $win.5.1.f -side left -fill x -padx 1
    pack $win.5.1.e -side left -fill x -padx 3
    pack $win.5.1.s -side left -fill x -padx 0

    label  $win.5.2.f -text "Blocked: "  -font $bfnt -width 10
    label  $win.5.2.e -text "" -font $fnt -width 40
    button $win.5.2.s -text "Select" -command {selectPDAcategories block} -font $bfnt
    pack $win.5.2.f -side left -fill x -padx 1
    pack $win.5.2.e -side left -fill x -padx 3
    pack $win.5.2.s -side left -fill x -padx 0
    fillSyncAdrSetBlockedCat
    
    tixScrolledWindow $win.6.w  -height 136
    $win.6.w subwidget hsb configure -width 12
    $win.6.w subwidget vsb configure -width 12
    pack $win.6.w -side left -fill x
    set swin [ $win.6.w subwidget window ]
    set syncAdrSelectAdrFieldBaseWin $swin
    for { set c 1 } { $c <= $syncAddrCustomMax } { incr c } {
	global syncAddrCustomField$c
	set cmd "set syncAddrCustomField$c \"\""
	eval $cmd
	frame $swin.$c
	pack $swin.$c -side top -fill x
	frame $swin.$c.m -relief flat
	label $swin.$c.m.l -text "User Field $c:" -width 12  -font $bfnt
	label $swin.$c.m.e -text "" -font $fnt -width 36
	button $swin.$c.m.b -text "Select" -command { syncAdrSelectAdrField } -font $bfnt
	bind $swin.$c.m.b <ButtonPress> {
	    set syncAdrSelectAdrFieldWin %W 
	}
	pack $swin.$c.m -side left -fill x -padx 4
	pack $swin.$c.m.l $swin.$c.m.e $swin.$c.m.b  -side left -fill x

    }

    label $win.7.l -text "" -font $fnt10
    label $win.8.l -text "Row tags in descriptions" -font $bfnt10
    tixScrolledHList $win.9.l -height 132 -width 484 -options { \
	hlist.header  true
	hlist.columns 3
    }

    set adrSyncHLstyle  [tixDisplayStyle text  -refwindow $win.9.l \
			     -fg black -anchor c \
			     -padx 8 -pady 2 -font $bfnt12]
    set adrSyncHLstyle2 [tixDisplayStyle text  -refwindow $win.9.l \
			     -fg black -anchor c \
			     -padx 8 -pady 2 -font $bfnt12 -fg red2 ]

    pack $win.7.l $win.8.l $win.9.l -side  left -fill x
    $win.9.l subwidget hlist header create 0 -itemtype text -relief raised  -text "Tag" -style $adrSyncHLstyle
    $win.9.l subwidget hlist header create 1 -itemtype text -relief raised  -text "Address Field" \
	-style $adrSyncHLstyle
    $win.9.l subwidget hlist header create 2 -itemtype text -relief raised  -text "X"  -style $adrSyncHLstyle2
    $win.9.l subwidget hlist column width 0 256
    $win.9.l subwidget hlist column width 1 188
    $win.9.l subwidget hlist column width 2 32
    set hwin [$win.9.l subwidget hlist]
    bind $hwin <Button-1> {syncAdrSelectAdrInfoTag %x}
    
}
proc fillSyncAddrTagsList { } {
    global syncAddrTagsList syncAdrBaseWin fnt12

    set win [$syncAdrBaseWin.9.l subwidget hlist]
    set adrSyncHLstyle [tixDisplayStyle text  -refwindow $syncAdrBaseWin.9.l \
			     -fg black -anchor c \
			     -padx 8 -pady 2 -font $fnt12 -fg black ]
    set adrSyncHLstyle2 [tixDisplayStyle text  -refwindow $syncAdrBaseWin.9.l \
			     -fg black -anchor c \
			     -padx 8 -pady 2 -font $fnt12 -fg red2 ]
    $win delete all
    foreach ent $syncAddrTagsList {
	set tag [ lindex $ent 0]
	set field [ lindex $ent 1]
	$win add $tag -itemtype text -text $tag -style $adrSyncHLstyle
	$win item create $tag 1 -itemtype text  -text $field  -style $adrSyncHLstyle
	$win item create $tag 2 -itemtype text  -text "X" -style $adrSyncHLstyle2
    }
}
proc syncAdrSelectAdrInfoTag { xk } {
    global syncAddrTagsList syncAdrBaseWin

    if { $xk < 444 } {
	_syncAdrSelectAdrField 1
    } else { 
	after 250 syncAdrRemoveTagListEntry
    }
}
proc syncAdrRemoveTagListEntry { } {
    global syncAddrTagsList syncAdrBaseWin

    set win [$syncAdrBaseWin.9.l subwidget hlist]
    set selection  [$win selection get]
    set tagList ""
    foreach ent $syncAddrTagsList {
	if { [lindex $ent 0] != $selection } {
	    lappend tagList $ent
	}
    }
    set syncAddrTagsList $tagList
    fillSyncAddrTagsList
}
proc syncAdrSelectAdrField { } {
    _syncAdrSelectAdrField 0
}
proc _syncAdrSelectAdrField { tag } {
    global bfnt12 fnt12 bfnt fnt
    global syncAdrFields
    global adrSyncTagNameTag

    toplevel .sasaf
    
    frame .sasaf.d -bd 1 -relief raised
    frame .sasaf.b -relief flat
    pack .sasaf.d .sasaf.b -side top -fill x
    
    frame .sasaf.d.0 -relief flat
    frame .sasaf.d.1 -relief flat
    frame .sasaf.d.2 -relief flat
    pack .sasaf.d.0 -fill x -side top -pady 2
    pack .sasaf.d.1  .sasaf.d.2 -fill x  -side top

    if { $tag == 1 } {
	label .sasaf.d.0.l -text "Tag" -font $bfnt12 -width 6
	entry .sasaf.d.0.e -textvariable adrSyncTagNameTag -font $fnt12 -width 20
	pack .sasaf.d.0.l .sasaf.d.0.e -side left -fill x 
    }

    label .sasaf.d.1.l -text "Address Fields" -font $bfnt12
    pack .sasaf.d.1.l -side top  -fill x

    if { $tag == 1 } {
	tixScrolledListBox .sasaf.d.2.l -anchor w -width 200 -height 240 -scrollbar y \
	    -command adrSyncSelectTagItem
    } else {
	tixScrolledListBox .sasaf.d.2.l -anchor w -width 200 -height 240 -scrollbar y  \
	    -command adrSyncSelectCustomItem
    }
    .sasaf.d.2.l subwidget vsb configure -width 12
    .sasaf.d.2.l subwidget listbox configure -font $fnt12
    .sasaf.d.2.l subwidget listbox configure -selectmode single
    .sasaf.d.2.l subwidget listbox configure -selectbackground goldenrod2
    
    foreach fid [lsort -integer [ array names syncAdrFields] ] {
	.sasaf.d.2.l subwidget listbox insert end $syncAdrFields($fid)
    }
    pack .sasaf.d.2.l -side top -fill x

    if { $tag == 1 } {
	button .sasaf.b.ok -text "O.K." -command { adrSyncSelectTagItem
	    destroy .sasaf}
    } else {
	button .sasaf.b.ok -text "O.K." -command { adrSyncSelectCustomItem
	    destroy .sasaf}
    }
    button .sasaf.b.can -text "Cancel" -command { destroy .sasaf}
	
    pack .sasaf.b.ok -side right 
    pack .sasaf.b.can -side left

    if { $tag == 1 } {
	centerWin .sasaf 202 316
    } else {
	centerWin .sasaf 202 296
    }
}
proc adrSyncSelectCustomItem { } {
    global syncAdrSelectAdrFieldWin syncAdrSelectAdrFieldBaseWin


    if { [string first $syncAdrSelectAdrFieldBaseWin $syncAdrSelectAdrFieldWin] == 0 } {
	regsub $syncAdrSelectAdrFieldBaseWin $syncAdrSelectAdrFieldWin {} varWin
	set varid [ lindex [split $varWin .] 1]
	global syncAddrCustomField$varid
	
	set selection  [.sasaf.d.2.l  subwidget listbox curselection]
	set syncAddrCustomField$varid [lindex [.sasaf.d.2.l  subwidget listbox get $selection $selection] 0]
	set cmd "$syncAdrSelectAdrFieldBaseWin.$varid.m.e configure -text \$syncAddrCustomField$varid"
	eval $cmd
    }
}
proc adrSyncSelectTagItem { } {
    global adrSyncTagNameTag syncAddrTagsList
    
    if {$adrSyncTagNameTag != "" } {
	set new 1
	foreach ent $syncAddrTagsList {
	    if { [lindex $ent 0 ] == $adrSyncTagNameTag } {
		set new 0
	    }
	}
	if { $new == 1 } {
	    set selection  [.sasaf.d.2.l  subwidget listbox curselection]
	    if { $selection != ""} {
		set item [lindex [.sasaf.d.2.l  subwidget listbox get $selection $selection] 0]
		set ent "$adrSyncTagNameTag {$item}"
		lappend syncAddrTagsList $ent
		fillSyncAddrTagsList
	    }
	}
    }
}

proc selectPDAcategories { mode } {
    global  pdaAdrCategories fnt12 fnt bfnt pdaAdrCatMode syncAddrBlockedCategory syncAddrDefCategory


    set pdaAdrCatMode $mode
    toplevel .spc 
    frame .spc.f -relief raised -bd 1
    frame .spc.b 
    pack .spc.f .spc.b -side top -fill x

    tixScrolledListBox .spc.f.l -width 200 -height 200  
    .spc.f.l subwidget listbox configure -selectbackground goldenrod2
    .spc.f.l subwidget listbox configure -font $fnt12 
    if { $mode == "def" } {
	.spc.f.l subwidget listbox configure   -selectmode single
	wm  title .spc "Select Default Category"
	foreach cat $pdaAdrCategories {
	    .spc.f.l subwidget listbox insert end $cat
	    if { [lsearch $syncAddrDefCategory $cat] >= 0 } {
		.spc.f.l subwidget listbox  selection set end 
	    }
	}
    } else {
	.spc.f.l subwidget listbox configure -selectmode extended
	wm  title .spc "Select Blocked Categories"
	foreach cat $pdaAdrCategories {
	    .spc.f.l subwidget listbox insert end $cat
	    if { [lsearch $syncAddrBlockedCategory $cat] >= 0 } {
		.spc.f.l subwidget listbox  selection set end 
	    }
	}
   }
    pack .spc.f.l  -side top -fill x
    
    button .spc.b.ok -text "O.K." -font $bfnt -command {
	if { $pdaAdrCatMode  == "def" } {
	    syncAdrSetDefCat
	} else { 
	    syncAdrSetBlockedCat
	}
	destroy .spc
    }
    button .spc.b.can -text "Cancel"  -font $fnt -command {destroy .spc   }

    pack .spc.b.ok -side right
    pack .spc.b.can -side left
    bind .spc.b.ok <Return> {
	if { $pdaAdrCatModee  == "def" } {
	    syncAdrSetDefCat
	} else { 
	    syncAdrSetBlockedCat
	}
	destroy .spc
    }
    bind .spc.b.ok <KP_Enter> {
	if { $pdaAdrCatMode  == "def" } {
	    syncAdrSetDefCat
	} else { 
	    syncAdrSetBlockedCat
	}
	destroy .spc
    }
    centerWin .spc 204 236
}

proc syncAdrSetDefCat { } {
    global syncAddrDefCategory syncAdrBaseWin
    
    set selection [ .spc.f.l subwidget listbox curselection]
    set syncAddrDefCategory [.spc.f.l subwidget listbox get $selection ]
    $syncAdrBaseWin.5.1.e configure -text $syncAddrDefCategory
}
proc syncAdrSetBlockedCat { } {
    global syncAddrBlockedCategory fnt syncAdrBaseWin 

    set syncAddrBlockedCategory ""
    set selection [ .spc.f.l subwidget listbox curselection]
    foreach sel $selection {
	set ent [.spc.f.l subwidget listbox get $sel ]
	set syncAddrBlockedCategory "$syncAddrBlockedCategory  {$ent}"
	set txt "$txt,  $ent"
    }
    fillSyncAdrSetBlockedCat
}
proc fillSyncAdrSetBlockedCat { } {
    global syncAddrBlockedCategory fnt syncAdrBaseWin 

    set txt ""
    foreach cat $syncAddrBlockedCategory {
	set txt "$txt,  $cat"
    }	
    set txt [string trim $txt ", "]
    set w [expr [font measure $fnt "0"] * 40 ]
    if { [font measure $fnt $txt] > $w } {
	set txt2 "$txt ..."
	set l [string length $txt]
	while { [font measure $fnt $txt2] > $w } {
	    incr l -1
	    set txt [string range $txt 0 $l]
	    set txt2 "$txt ..."
	}
	set txt $txt2
    }
    $syncAdrBaseWin.5.2.e configure -text $txt
}
proc syncAdrSetDefFolder { } {
    global syncAddrDefFolder syncAdrBaseWin adlList idx2dirid adrDirList
	
    set syncAddrDefFolder [ adrDirPath2DirName  [ .adl.t.l subwidget hlist info selection] ]
}
proc syncAdrSetBlockedFolders { } {
    global syncAddrBlockedFolder fnt syncAdrBaseWin 

    set syncAddrBlockedFolder ""
    set txt ""
    set selection [ .adl.t.l subwidget hlist info selection]
    foreach sel $selection {
	set dir [ adrDirPath2DirName  $sel ]
	set syncAddrBlockedFolder "$syncAddrBlockedFolder  {$dir}"
    }
    fillSyncAdrSetBlockedFolders
}
proc adrDirPath2DirName { path } {
    global adrDirList
    set list [split  $path .]
    set obj  [lindex $list end]
    if { $obj == "" } {
	set folder "-"
    } else {
	set idx [trimLead0 [string range   $obj  3 end ]]
	set folder [ lindex [lindex $adrDirList $idx] 1]
    }
    return $folder
}
proc fillSyncAdrSetBlockedFolders { } {
    global syncAddrBlockedFolder fnt syncAdrBaseWin

    set txt ""
    foreach fol $syncAddrBlockedFolder {
	set txt "$txt,  $fol"
    }	
    set txt [string trim $txt ", "]
    set w [expr [font measure $fnt "0"] * 40 ]
    if { [font measure $fnt $txt] > $w } {
	set txt2 "$txt ..."
	set l [string length $txt]
	while { [font measure $fnt $txt2] > $w } {
	    incr l -1
	    set txt [string range $txt 0 $l]
	    set txt2 "$txt ..."
	}
	set txt $txt2
    }
    $syncAdrBaseWin.3.2.e configure -text $txt
}



proc selectSyncPDAdevice { calWin projWin adrWin dev } {
    global syncDeviceParameter syncAddrCustomMax
    global syncReadCmdStr syncWriteCmdStr syncCsvStr syncCsvSep
    global syncConnectCmdStr syncDisConnCmdStr syncConnectCmdMsg  syncConnCheckCmdStr syncConnectCmdHelp
    global syncInFile syncCatFile syncOutFile syncStartLine
    global syncAddrCustomCount 
    global syncAddrDefFolder  syncAddrDefCategory syncAdrFields
    global syncAddrCustomMax 
    global syncAddrTagsList

    if { $syncDisConnCmdStr != "" } {
	set cmd [syncGetCommandStr $syncDisConnCmdStr ]
	catch {eval $cmd }
    }


    set syncConnectCmdStr      [lindex $syncDeviceParameter($dev) 0]
    set syncConnectCmdMsg      [lindex $syncDeviceParameter($dev) 1]
    set syncConnectCmdHelp     [lindex $syncDeviceParameter($dev) 2]
    set syncConnCheckCmdStr    [lindex $syncDeviceParameter($dev) 3]
    set syncDisConnCmdStr      [lindex $syncDeviceParameter($dev) 4]
    set syncReadCmdStr(adr)    [lindex $syncDeviceParameter($dev) 5]
    set syncWriteCmdStr(adr)   [lindex $syncDeviceParameter($dev) 6]
    set syncCsvStr(adr)        [lindex $syncDeviceParameter($dev) 7]
    set syncCsvSep(adr)        [lindex $syncDeviceParameter($dev) 8]
    set syncAddrDefCategory    [lindex $syncDeviceParameter($dev) 9]
    set syncAddrDefFolder      [lindex $syncDeviceParameter($dev) 10]
    set syncStartLine(adr)     [lindex $syncDeviceParameter($dev) 11]
    set syncAddrCustomCount    [lindex $syncDeviceParameter($dev) 12]

    set syncInFile [format "/tmp/minkowsky.sync.input.%d.csv" [pid] ]
    set syncCatFile [format "/tmp/minkowsky.sync.cats.%d.csv" [pid] ]
    set syncOutFile [format "/tmp/minkowsky.sync.output.%d.csv" [pid] ]

#     puts ">>>> $syncDeviceParameter($dev)"
#     puts "---> $syncConnectCmdStr"
#     puts "---> $syncConnectCmdMsg"
#     puts "---> $syncConnectCmdHelp"
#     puts "---> $syncConnCheckCmdStr"
#     puts "---> $syncDisConnCmdStr"

#    set syncAddrCustomCount [lindex $syncDeviceParameter($dev) 0 ]

    set swin [ $adrWin.6.w subwidget window ]
    for { set c 1 } { $c <= $syncAddrCustomMax } { incr c } {
	if { $c <= $syncAddrCustomCount } {
	    $swin.$c.m.b configure -state normal
	    $swin.$c.m.l configure -fg black    
	    $swin.$c.m.e configure -fg black    
	} else { 
	    $swin.$c.m.b configure -state disabled
	    $swin.$c.m.l configure -fg grey35
	    $swin.$c.m.e configure -fg grey35   
	}
    }

    syncGetCategories
}


proc check4PDAdialog { txt } {
    global pdaAvail syncDevice fnt bfnt 
    global syncConnectCmdStr syncDisConnCmdStr syncConnectCmdMsg

    set cmd [syncGetCommandStr $syncConnectCmdStr ]
    eval $cmd 


    set pdaAvail 0
    toplevel .c4p 
    wm title .c4p "Warning: PDA must be connected"

    frame .c4p.m -relief raised -bd 1
    frame .c4p.b
    pack .c4p.m  -side top -fill both   -expand 1
    pack .c4p.b  -side bottom -fill x
    message .c4p.m.m -text "Please make sure that your PDA ($syncDevice) is connected. $txt.\n$syncConnectCmdMsg" -font $fnt -width 360
    pack .c4p.m.m -side top -fill x

    button .c4p.b.ok -text "O.K." -font $bfnt  -command {
	set pdaAvail 1
	destroy .c4p
    }
    button .c4p.b.hlp -text "Help" -font $fnt -command {
	syncConnectHelpDialog
    }
    button .c4p.b.can -text "Cancel" -font $fnt  -command {
	set cmd [syncGetCommandStr $syncDisConnCmdStr ]
	eval $cmd 
	destroy .c4p
    }
    pack .c4p.b.ok -side right
    pack .c4p.b.hlp -side right -padx 16
    pack .c4p.b.can -side left

    bind .c4p.b.ok <Return> {
	set pdaAvail 1
	destroy .c4p
    }
    bind .c4p.b.ok <KP_Enter> {
	set pdaAvail 1
	destroy .c4p
    }
    centerWin .c4p 368 188
    wm transient .c4p .sync
    grab .c4p
}
proc syncConnectHelpDialog { } {
    global syncConnectCmdHelp fnt bfnt

    toplevel .sch

    wm title .sch "Help: How to connect your PDA"

    frame .sch.m -relief raised -bd 1
    frame .sch.b
    pack .sch.m  -side top -fill both -expand 1
    pack .sch.b  -side bottom -fill x
    if { $syncConnectCmdHelp == "" } {
	message .sch.m.m -text "Sorry there is no help text for this PDA." -font $fnt -width 300	
    } else {
	message .sch.m.m -text $syncConnectCmdHelp -font $fnt -width 320
    }
    pack .sch.m.m -side top -fill x

    button .sch.b.ok -text "O.K." -font $bfnt  -command {destroy .sch   }
    bind .sch.b.ok <Return> {destroy .sch   }
    bind .sch.b.ok <KP_Enter> {destroy .sch   }
    pack .sch.b.ok -side right

    centerWin .sch 328 240
    wm transient .sch .c4p 
    grab .sch
}
proc syncGetCategories { } {
    global pdaAvail pdaAdrCategories syncReadCmdStr syncCatFile syncDevice

    check4PDAdialog "Minkowsky needs to read the list of categories"
    update
    after 250
    tkwait window .c4p


    if { $pdaAvail == 0 } {
	destroy .sync
	return
    }
    
    openMeter "Checking connnection to $syncDevice" "checking connection" 0
    after 500

    
 
    if { ! [checkConnection 10 ] } {
	closeMeter 
	update
	_ErrorDialog "Error: No Connection" "There is no connection to the PDA"
	destroy .sync
    } else {
	updateMeter 84 "Getting Categories"
	set cmd [syncGetReadCommand $syncReadCmdStr(adr)]
	eval $cmd
	
	updateMeter 96 "Reading Categories"
	set pdaAdrCategories ""
	set fid [ open $syncCatFile "r"]
	while { ! [eof $fid] } {
	    set cat [string trim [gets $fid]]
	    if { $cat != "" } { lappend pdaAdrCategories $cat }
	}
	updateMeter 100 "done"
	after 250
	closeMeter
    }
}


proc doSync { } {
    global syncDirection username
    global syncReadCmdStr syncWriteCmdStr syncCsvStr  syncCsvSep
    global syncInFile syncCatFile syncOutFile  syncStartLine
    global syncAddrCustomMax syncAddrDefFolder syncAddrDefCategory
    global syncAddrTagsList syncAddrBlockedFolder syncAddrBlockedCategory


    busyCursor .sync 
    .sync.b.abb  configure -state disabled 
    .sync.b.sync configure -state disabled 
    .sync.b.pref configure -state disabled 
    update
 
    openMeter "Syncronisation" "start sync" 1
    set ret [transfer openSync $username]
    if { $ret == "Error" } { return }
    if { $ret == "" } { return }
    updateMeter 2 "start sync done"
    
    #puts "$$$$$ sync opened"
    set partName(adr) "Addresses"
    set partName(prj) "Projects"
    set partName(cal) "Calendar"
    set pi 0
    foreach part [array names syncDirection] {
	#puts "$$$$$ sync on $part : $syncDirection($part)"

	updateMeter [expr 2+ $pi*32] "sync $partName($part)"

	if { $syncDirection($part) != "off" } {
	    #puts "$$$$ starting sync of $part"
	    
	    set cmd [syncGetReadCommand $syncReadCmdStr($part)]
	    set ret [ catch {eval $cmd} ]
	    if { $ret != 0 } { 
		_ErrorDialog "Can't read from PDA" "Can't read data ($part) from the PDA"
		return
	    }
	    #puts "$$$$$ sync read PDA data done"

	    #puts "$$$$$ sync counting on $syncInFile"
	    set entryCount 0
	    set fid [open $syncInFile "r" ]
	    while { ! [eof $fid] } {
		set line [gets $fid]
		incr entryCount
	    }
	    close $fid
	    #puts "$$$$$ $entryCount line from PDA "
	    if { $part == "adr" } {
		set customList ""
		for { set c 1 } { $c <= $syncAddrCustomMax } { incr c } {
		    global syncAddrCustomField$c
		    set cmd "set customList \"\$customList {\[string trim \$syncAddrCustomField$c \]}\""
		    eval $cmd
		}
		set customList [string trim $customList]
		set tagList ""
		foreach ent $syncAddrTagsList { set tagList "$tagList [lindex $ent 0] [lindex $ent 1]" }
		set tagList [string trim $tagList]

		#puts "$$$$$ sync init :"
		set ret [ transfer initSync $username $syncDirection($part) $part \
			      $syncCsvStr($part) $syncCsvSep($part) $entryCount \
			      $syncAddrDefFolder $syncAddrDefCategory \
			      $tagList $customList \
			      $syncAddrBlockedFolder $syncAddrBlockedCategory]
		if { $ret == "" || $ret == "Error" } { return}
		after 1000
	    } elseif { $part == "prj" } {
	    } elseif { $part == "cal" } {
	    } 
	    if { $ret == "Error" } { return }
	    #puts "$$$$$ sync init done"


	    updateMeter [expr 3+ $pi*32] "sync in $partName($part) "

	    set fid [open $syncInFile "r" ]

	    set count 1
	    while { ! [eof $fid] } {
		set line [string  trim [gets $fid] ]
		if { $count > $syncStartLine($part) } {
		    if { $line != "" } {
			set ret [ transfer syncInto $username  $line]
			if { $ret == "" || $ret == "Error" } { return }
			#puts "$count: sending $line"
		    }
		}
		incr count
		updateMeter [expr 3+ $pi*32 + int(floor(15*double($count)/double($entryCount)))] \
		    "sync in $partName($part) "

		update
	    }
	    close $fid
	    set ret [ transfer  syncIntoStop $username ]
	    if { $ret == "" || $ret == "Error" } { return }
	    #puts "$$$$$ sync syncInStop done"

	    
	    if { $syncDirection($part) != "into" } {
		updateMeter [expr 19+ $pi*32] "sync out $partName($part) "
		set fid [open $syncOutFile "w" ]
		set done 0
		set count 0
		while {$done == 0 } {
		    set line [ request nextCsv $username] 
		    if { $ret == "Error" } { return }
		    if { $ret == "" } { return }
		    if { $line != "done" } {
			puts $fid $line
		    } else {
			break
		    }
		    incr count
		    set progress [expr int(floor(15*double($count)/double($entryCount)))]
		    if { $progress > 15} { set progress 15}
		    updateMeter [expr 19+ $pi*32 +$progress] "sync out $partName($part) "

		    update
		}
		close $fid
		
		
		set cmd [syncGetWriteCommand $syncWriteCmdStr($part)]
#		puts "writeCMD; $cmd"
		set ret [ catch {eval $cmd} ]
		if { $ret != 0 } { 
		    _ErrorDialog "Can't writeto PDA" "Can't write data ($part) on the PDA"
		    return
		}
	    }
	    set ret [ transfer endSync $username]
	    if { $ret == "" || $ret == "Error" } { return }
	    #puts "$$$$$ sync endSync done"
	}
	updateMeter [expr 34+ $pi*32] "sync $partName($part) done"
	incr pi
    }
    updateMeter 98 "sync  done"
    set ret [ transfer closeSync $username]
    if { $ret == "" || $ret == "Error" } { return }
    updateMeter 100 "sync closed"
    fillAddress
    updateMeter 100 "data reloaded"
    .sync.b.abb  configure -state normal
    .sync.b.sync configure -state normal
    .sync.b.pref configure -state normal
    after 250
    closeMeter
    stdCursor .sync 
}

proc checkConnection { withMeter } {
    global syncConnCheckCmdStr
    
    set met 0
    set retries 8
    set cmd [syncGetReadCommand $syncConnCheckCmdStr]
#     puts "cmd=$cmd"
    while { $retries >= 0 } {
	set ret [catch {eval $cmd} ]
# 	puts "check connection : $ret"
	if { $ret == 0 } { return 1 }
	if { $withMeter >0 } { 
	    set met [expr $met + $withMeter ]
	    updateMeter $met "check failed, retry [expr 9-$retries]"
	}
	after 250
	incr retries -1
    }
    return 0
}

proc syncGetCommandStr {cmdStr  } {
    global syncDev  clientBinPath

#     puts "SYNC CONNECT: $cmdStr < $clientBinPath"

    regsub {MINKO_PATH} $cmdStr $clientBinPath tmp
    regsub {%d} $tmp $syncDev cmd
    set cmd "exec $cmd"

#     puts "-> $cmd"
    return $cmd
}

proc syncGetReadCommand { cmdStr } {
    global syncDev syncInFile syncCatFile clientBinPath
    

    regsub {MINKO_PATH} $cmdStr $clientBinPath tmp2
    regsub {%d} $tmp2 $syncDev tmp
    regsub {%f} $tmp $syncInFile tmp2
    regsub {%F} $tmp2 $syncCatFile cmd
    set cmd "exec $cmd"
    return $cmd
}
proc syncGetWriteCommand { cmdStr } {
    global syncDev syncOutFile clientBinPath
    

    regsub {MINKO_PATH} $cmdStr $clientBinPath tmp2
    regsub {%d} $tmp2 $syncDev tmp
    regsub {%f} $tmp $syncOutFile cmd
    set cmd "exec $cmd"
    return $cmd
}


proc saveSyncPrefs { } {
    global syncDirection username
    global syncReadCmdStr syncWriteCmdStr syncCsvStr  syncCsvSep
    global syncInFile syncCatFile syncOutFile  syncStartLine
    global syncAddrCustomMax syncAddrDefFolder syncAddrDefCategory
    global syncAddrTagsList syncAddrBlockedFolder syncAddrBlockedCategory
    global username syncDevice syncDevi

    set syncPref "{$syncDevice} $syncDevi"


    set syncCalPref ""


    set syncPrjPref "" 


    set customList ""
    for { set c 1 } { $c <= $syncAddrCustomMax } { incr c } {
	global syncAddrCustomField$c
	set cmd "set customList \"\$customList {\[string trim \$syncAddrCustomField$c \]}\""
	eval $cmd
    }
    set customList [string trim $customList]
    set syncAdrPref "$syncDirection(adr) $syncAddrDefFolder $syncAddrDefCategory {$syncAddrBlockedFolder} {$syncAddrBlockedCategory} {$syncAddrTagsList} {$customList}"


    set ret [transfer saveSyncPrefs $username $syncPref $syncCalPref $syncPrjPref $syncAdrPref]
}



proc readSyncPrefs { } {
    global syncDirection username
    global syncReadCmdStr syncWriteCmdStr syncCsvStr  syncCsvSep
    global syncInFile syncCatFile syncOutFile  syncStartLine
    global syncAddrCustomMax syncAddrDefFolder syncAddrDefCategory
    global syncAddrTagsList syncAddrBlockedFolder syncAddrBlockedCategory
    global username syncDevice syncDevi

    global syncAdrBaseWin syncAdrSelectAdrFieldBaseWin

    set ret [request getSyncPrefs $username]
    if { $ret == "" || $ret == "Error" } { return }

#    set ret ""

    set list [lindex $ret 0]
    if { [llength $list ] > 1 } {
	set $syncDevice [lindex $list 0]
	set $syncDevi   [lindex $list 1]
    }

    selectSyncPDAdevice [.sync.n.n subwidget cal] [.sync.n.n subwidget proj] [.sync.n.n subwidget addr] $syncDevice
    update
    if { ! [ winfo exists .sync] } { return}
#    return

    set list [lindex $ret 1]
    set $syncDirection(cal) [lindex $list 0]

    set list [lindex $ret 2] 
    set $syncDirection(prj) [lindex $list 0]

    set list [lindex $ret 3] 
    set syncDirection(adr)      [lindex $list 0]
    set syncAddrDefFolder       [lindex $list 1]
    set syncAddrDefCategory     [lindex $list 2]
    set syncAddrBlockedFolder   [lindex $list 3]
    set syncAddrBlockedCategory [lindex $list 4]
    set syncAddrTagsList        [lindex $list 5]
    set customList              [lindex $list 6]
    for { set c 1 } { $c <= $syncAddrCustomMax } { incr c } {
	global syncAddrCustomField$c
	set syncAddrCustomField$c ""
    }
    set c 1
    foreach ent $customList {
	set syncAddrCustomField$c $ent
	set cmd "$syncAdrSelectAdrFieldBaseWin.$c.m.e configure -text \$syncAddrCustomField$c"
	eval $cmd
	incr c
	if { $c > $syncAddrCustomMax } { break }
    }

    $syncAdrBaseWin.5.1.e configure -text $syncAddrDefCategory
    fillSyncAdrSetBlockedCat
    fillSyncAdrSetBlockedFolders
    fillSyncAddrTagsList
}


###########################################################################

# set syncDeviceList "{Agenda VR3} {Palm} {funny 2} {funny 7}"
# set dev "Agenda VR3"
# set syncDeviceParameter($dev) "4"
# set dev "Palm"
# set syncDeviceParameter($dev) "4"
# set dev "funny 2"
# set syncDeviceParameter($dev) "2"
# set dev "funny 7"
# set syncDeviceParameter($dev) "7"
# set syncDirection(adr) "into"
# set syncDirection(prj) "off"
# set syncDirection(cal) "off"
# set syncAddrDefFolder   "Import"
# set syncAddrDefCategory "Other"
# set syncAdrFields(-1) "empty"
# set syncAdrFields(60) "Kontoinhaber (Konto 1)"
# set syncAdrFields(62) "Bank         (Konto 1)"
# set syncAdrFields(63) "BLZ          (Konto 1)"
# set syncAdrFields(61) "Kontonummer  (Konto 1)"
# set syncAdrFields(72) "Geburtstag"



